# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from nltk.parse.corenlp import CoreNLPParser
from nlpkit.examples.example_corpus import corpus
from nlpkit.nlp_feature_extraction import CFGExtractor


if __name__ == "__main__":
    sf_parser = CoreNLPParser(url='http://localhost:9000/')

    # CFGExtractor
    cfg_extractor = CFGExtractor(sf_parser)
    X = cfg_extractor.fit_transform(corpus)

    print(X)

    # Example usage in a pipeline
    # pipeline = Pipeline([
    #     ('cfg', CFGExtractor(parser=self._sf_parser, include_lexical=True)),
    #     ('vec', TfidfVectorizer(preprocessor=dummy_tokenizer, tokenizer=dummy_tokenizer0)),
    #     ('clf', BernoulliNB())
    # ])



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from unittest import mock
import pytest

from nlpkit.nlp_feature_extraction.syntax_features import CFGExtractor
import nlpkit.nlp_feature_extraction.tests.test_data.doc_test_data as test_data
import nlpkit.nlp_feature_extraction.tests.test_data.nlp_test_data as nlp_test_data


@pytest.fixture(scope="module")
def mockParser():
    mock_parser = mock.Mock()
    mock_parser.raw_parse.side_effect = tag_side_effect
    return mock_parser


def tag_side_effect(*args, **kwargs):
    if args[0] == test_data.DOC_1:
        return iter(nlp_test_data.PARSE_TREE_NLTK_DOC_1)
    elif args[0] == test_data.DOC_2:
        return iter(nlp_test_data.PARSE_TREE_NLTK_DOC_2)


def test_cfg_extractor_nonlexical(mockParser):
    expected_results = [['ROOT -> S',
                         'S -> NP VP .',
                         'NP -> DT JJ JJ NN',
                         'VP -> VBD PP',
                         'PP -> IN NP',
                         'NP -> DT JJ NN'],
                        ['ROOT -> S',
                         'S -> NP ADVP VP .',
                         'NP -> PRP$ NN',
                         'ADVP -> RB',
                         'VP -> VBZ NP',
                         'NP -> JJ NN']]

    cfg_extractor = CFGExtractor(mockParser, include_lexical=False)
    assert cfg_extractor.transform(test_data.DOCS) == expected_results


def test_cfg_extractor_lexical(mockParser):
    expected_results = [['ROOT -> S',
                         'S -> NP VP .',
                         'NP -> DT JJ JJ NN',
                         "DT -> 'The'",
                         "JJ -> 'quick'",
                         "JJ -> 'brown'",
                         "NN -> 'fox'",
                         'VP -> VBD PP',
                         "VBD -> 'jumped'",
                         'PP -> IN NP',
                         "IN -> 'over'",
                         'NP -> DT JJ NN',
                         "DT -> 'the'",
                         "JJ -> 'lazy'",
                         "NN -> 'dog'",
                         ". -> '.'"],
                        ['ROOT -> S',
                         'S -> NP ADVP VP .',
                         'NP -> PRP$ NN',
                         "PRP$ -> 'My'",
                         "NN -> 'dog'",
                         'ADVP -> RB',
                         "RB -> 'also'",
                         'VP -> VBZ NP',
                         "VBZ -> 'likes'",
                         'NP -> JJ NN',
                         "JJ -> 'eating'",
                         "NN -> 'sausage'",
                         ". -> '.'"]]

    cfg_extractor = CFGExtractor(mockParser, include_lexical=True)
    assert cfg_extractor.transform(test_data.DOCS) == expected_results



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>

University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from sklearn.base import BaseEstimator, TransformerMixin
from textstat.textstat import textstat as ts


class TextStatsExtractor(BaseEstimator, TransformerMixin):
    """
    Calculates various text statistics and readability scores for a collection of text documents.
    """

    def transform(self, X, y=None):
        """
        :param X: A collection of docs
        :param y:
        :return: a list of dictionaries with calculated scores for each doc
        """
        return self._calculate_scores(X)

    def fit(self, X, y=None):
        return self

    def _calculate_scores(self, docs):
        docs_scores = []

        for doc in docs:
            scores = {}
            scores['chars'] = ts.char_count(doc)
            scores['words'] = ts.lexicon_count(doc)
            scores['sents'] = ts.sentence_count(doc)
            #scores['syllables'] = ts.syllable_count(doc)
            scores['avg_sent_length'] = ts.avg_sentence_length(doc)
            scores['avg_syllables_per_word'] = ts.avg_syllables_per_word(doc)
            scores['avg_letters_per_word'] = ts.avg_letter_per_word(doc)
            scores['flesch'] = ts.flesch_reading_ease(doc)
            #scores['smog'] = ts.smog_index(doc)
            #scores['coleman_liau'] = ts.coleman_liau_index(doc)
            scores['automated_readability'] = ts.automated_readability_index(
                doc)
            #scores['linsear'] = ts.linsear_write_formula(doc)
            #scores['difficult_words'] = ts.difficult_words(doc)
            scores['dale_chall'] = ts.dale_chall_readability_score(doc)
            #scores['gunning_fog'] = ts.gunning_fog(doc)
            scores['lix'] = ts.lix(doc)
            docs_scores.append(scores)

        return docs_scores



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from .ner_features import NERPreprocessor
from .ner_features import NamedEntitiesCounter
from .word_embeddings_features import WordEmbedsDocVectorizer
from .pos_features import POSExtractor
from .pos_features import POSTagPreprocessor
from .syntax_features import CFGExtractor
from .liwc_features import LIWCExtractor
from .text_statistics_features import TextStatsExtractor



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

import os

from gensim.models.keyedvectors import KeyedVectors

from examples.example_corpus import corpus
from nlpkit.nlp_feature_extraction import WordEmbedsDocVectorizer

# Replace with the path of a pre-trained w2v model
WORD_EMBEDDINGS_FILEPATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '..', '..', 'fake-news', 'data', 'resources', 'word-embeddings',
                 'glove.6B', 'glove.6B.300d.word2vec.txt'))


if __name__ == "__main__":
    word2vec = KeyedVectors.load_word2vec_format(
        WORD_EMBEDDINGS_FILEPATH, binary=False)

    w2v_vectorizer = WordEmbedsDocVectorizer(word2vec, tfidf_weights=True)
    X = w2v_vectorizer.fit_transform(corpus)

    print(X)

    # Example usage in a pipeline
    # pipeline = Pipeline(
    #     ('vec', WordEmbedsDocVectorizer(word2vec, tfidf_weights=True)),
    #     ('clf', SVC(kernel='linear', C=1, probability=True))
    # ])



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

import collections
from sklearn.base import BaseEstimator, TransformerMixin


class POSExtractor(BaseEstimator, TransformerMixin):
    """
    Extracts Parts of Speech (POS) counts for a collection of text documents.
    """

    def __init__(self, tagger):
        """
        :param tagger: a StanfordPosTagger object
        """
        self.tagger = tagger

    def transform(self, X, y=None):
        """
        :param X: a collection of docs
        :param y:
        :return: a list of dictionaries (pos_tag -> raw counts) corresponding to documents
        """
        return self._count_pos(X, self.tagger)

    def fit(self, X, y=None):
        return self

    def _count_pos(self, docs, tagger):
        pos = []
        for doc in docs:
            # Pos tag doc tokens
            tags = tagger.tag(doc.split())

            # A Counter object to store the frequencies of pos tags in a doc
            pos_counter = collections.Counter()
            for tag in tags:
                pos_counter[tag[1]] += 1

            pos.append(pos_counter)
        return pos


class POSTagPreprocessor(BaseEstimator, TransformerMixin):
    """
    Pre-processes text documents by tagging each word in the form of word_TAG_
    e.g. what_WP. Can be used to generate POS tagged n-grams.
    """

    def __init__(self, tagger):
        """
        :param tagger: a StanfordPosTagger object
        """
        self.tagger = tagger

    def transform(self, X, y=None):
        """
        :param X: a collection of docs
        :param y:
        :return: a list of tagged docs
        """
        return self._pos_tag_docs(X, self.tagger)

    def fit(self, X, y=None):
        return self

    def _pos_tag_docs(self, docs, tagger):
        processed_docs = []
        for doc in docs:
            # Pos tag doc tokens
            tags = tagger.tag(doc.split())

            tagged_doc = []
            # tag each word in the form word_TAG_ e.g. what_WP
            for word_tag in tags:
                tagged_word = word_tag[0] + '_' + word_tag[1]
                tagged_doc.append(tagged_word)

            tagged_doc = ' '.join(tagged_doc)
            processed_docs.append(tagged_doc)

        return processed_docs



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from sklearn.base import BaseEstimator, TransformerMixin
from nltk import Tree


class CFGExtractor(BaseEstimator, TransformerMixin):
    """
    Extracts the Context Free Grammar (CFG) production rules found in a collection of text documents.
    """

    def __init__(self, parser, include_lexical=True):
        """
        :param parser: An StanfordParser object
        :param include_lexical: Option to include or exclude production rules that contain
                                terminal tokens (words)
        """
        self.parser = parser
        self.include_lexical = include_lexical

    def transform(self, X, y=None):
        """
        :param X: a collection of docs
        :param y:
        :return: a list of lists that contain the production rules found in each
                 document
        """
        return self._extract_cfg_rules(X, self.include_lexical)

    def fit(self, X, y=None):
        return self

    def _extract_cfg_rules(self, docs, include_lexical, print_progress=False):
        productions = []
        i = 0
        for doc in docs:
            # Parse doc and generate a syntax tree
            parse_tree = Tree("START", self.parser.raw_parse(doc))
            # Extract the productionvrules that correspond to the non-terminal
            # nodes of the tree
            prods = parse_tree.productions()

            # Create a list of all productions in document
            doc_prods = []
            for prod in prods:
                if (str(prod) == "START -> ROOT"):  # skip this
                    continue
                if (include_lexical):
                    doc_prods.append(str(prod))
                else:
                    # note that is_lexical is not the opposite of is_nonlexical
                    if (prod.is_nonlexical()):
                        doc_prods.append(str(prod))

            productions.append(doc_prods)

            # track progress
            if (print_progress):
                if (i % 500) == 0:
                    print("{0}/{1} docs processed...".format(i, len(docs)))
            i += 1

        return productions


def dummy_tokenizer(doc):
    """
    A pass through method for pre-tokenized documents. Use it as preprocessor,
    and tokenizer for the tfidf/count vectorizer. (to replace lambda x:x)
    :param doc:
    :return:
    """
    return doc



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from .liwc import Liwc

WORD_CAT_DICT_1 = {
    "love": [
        "affect",
        "posemo"
    ],
    "loved": [
        "affect",
        "posemo"
    ]
}

WORD_CAT_DICT_2 = {
    "abandon*": [
        "affect",
        "negemo"
    ],
    "absolute": [
        "cogmech"
    ],
    "love": [
        "affect",
        "posemo"
    ]
}


def test_build_trie():
    expected = {
        "l": {
            "o": {
                "v": {
                    "e": {
                        "$": [
                            "affect",
                            "posemo"
                        ],
                        "d": {
                            "$": [
                                "affect",
                                "posemo"
                            ]
                        }
                    }
                }
            }
        }
    }
    assert Liwc._build_char_trie(WORD_CAT_DICT_1) == expected


def test_build_trie_wildcard():
    expected = {
        "a": {
            "b": {
                "a": {
                    "n": {
                        "d": {
                            "o": {
                                "n": {
                                    "*": [
                                        "affect",
                                        "negemo"
                                    ],
                                    "$": [
                                        "affect",
                                        "negemo"
                                    ]
                                }
                            }
                        }
                    }
                },
                "s": {
                    "o": {
                        "l": {
                            "u": {
                                "t": {
                                    "e": {
                                        "$": [
                                            "cogmech"
                                        ]
                                    }
                                }
                            }
                        }
                    }
                }
            }
        },
        "l": {
            "o": {
                "v": {
                    "e": {
                        "$": [
                            "affect",
                            "posemo"
                        ]
                    }
                }
            }
        }
    }

    assert Liwc._build_char_trie(WORD_CAT_DICT_2) == expected


def test_search_trie():
    trie = Liwc._build_char_trie(WORD_CAT_DICT_1)

    assert Liwc._search_trie(trie, 'love') == ["affect", "posemo"]
    assert Liwc._search_trie(trie, 'loved') == ["affect", "posemo"]


def test_search_wildcard():
    trie = Liwc._build_char_trie(WORD_CAT_DICT_2)

    assert Liwc._search_trie(trie, 'abandon') == ["affect", "negemo"]
    assert Liwc._search_trie(trie, 'abandonment') == ["affect", "negemo"]



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

import collections
from sklearn.base import BaseEstimator, TransformerMixin


class NERPreprocessor(BaseEstimator, TransformerMixin):
    """
    Pre-processes text documents by replacing named entities with generic tags e.g.
        - PERSON
        - ORGANIZATION
        - LOCATION
    """

    def __init__(self, tagger):
        """
        :param tagger: A StanfordParser object
        """
        self.tagger = tagger

    def transform(self, X, y=None):
        """
        :param X: a list of documents
        :param y:
        :return: a list of documents with the named entities replaced by generic tags
        """
        return self._replace_entities(X, self.tagger)

    def fit(self, X, y=None):
        # Fit method is only required by scikit pipeline
        return self

    def _replace_entities(self, docs, tagger):
        processed_docs = []
        for doc in docs:
            ner_results = tagger.tag(doc.split())

            prev_entity = None
            tokens = []
            for token, entity_type in ner_results:
                if entity_type != 'O':
                    # By default "John Doe" will be replaced with PERSON PERSON
                    # If the the previous entity type matches current type they
                    # should merge to eg. PERSON
                    if prev_entity != entity_type:
                        # enclose in underscore to differentiate from words
                        entity_tag = '_' + entity_type + '_'
                        tokens.append(entity_tag)
                else:
                    tokens.append(token)
                prev_entity = entity_type

            processed_docs.append(' '.join(tokens))

        return processed_docs


class NamedEntitiesCounter(BaseEstimator, TransformerMixin):
    """
    Extracts Named Entity counts per entity type (e.g. PERSON) for a collection of text documents.
    """

    def __init__(self, tagger):
        """
        :param tagger: a CoreNLPParser object with tagtype='ner'
        """
        self.tagger = tagger

    def transform(self, X, y=None):
        """
        :param X: a collection of docs
        :param y:
        :return: a list of dictionaries (entity_type -> raw counts) corresponding to documents
        """
        return self._count_named_entities(X, self.tagger)

    def fit(self, X, y=None):
        return self

    def _count_named_entities(self, docs, tagger):
        ne = []
        for doc in docs:
            ner_results = tagger.tag(doc.split())

            # A Counter object to store the frequencies of pos tags in a doc
            ne_counter = collections.Counter()
            prev_entity = None
            for _, entity_type in ner_results:
                if entity_type == 'O':
                    ne_counter[entity_type] += 1
                else:
                    # fix for duplicate counts in case of (John, Person),
                    # (Doe,PERSON)
                    if prev_entity != entity_type:
                        ne_counter[entity_type] += 1
                prev_entity = entity_type

            ne.append(ne_counter)
        return ne



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from collections import Counter

from unittest import mock
import pytest

from nlpkit.nlp_feature_extraction.ner_features import NERPreprocessor, NamedEntitiesCounter
import nlpkit.nlp_feature_extraction.tests.test_data.nlp_test_data as nlp_test_data


@pytest.fixture(scope="module")
def mockTagger():
    mock_tagger = mock.Mock()
    mock_tagger.tag.side_effect = tag_side_effect
    return mock_tagger


def tag_side_effect(*args, **kwargs):
    if args[0] == nlp_test_data.NER_DOC_1.split(' '):
        return nlp_test_data.NER_TAGS_DOC_1
    elif args[0] == nlp_test_data.NER_DOC_2.split(' '):
        return nlp_test_data.NER_TAGS_DOC_2


def test_ner_preprocessor(mockTagger):
    expected_results = [
        "_PERSON_ agrees with _PERSON_ `` by voting to give _PERSON_ the benefit of the doubt on _COUNTRY_ . ''",
        'My dog also likes eating sausage .'
    ]

    ner_preprocessor = NERPreprocessor(mockTagger)
    assert ner_preprocessor.transform(
        nlp_test_data.NER_DOCS) == expected_results


def test_named_entities_counter(mockTagger):
    expected_results = [
        Counter({
            'O': 15,
            'PERSON': 3,
            'COUNTRY': 1
        }),
        Counter({'O': 7})
    ]

    named_entities_counter = NamedEntitiesCounter(mockTagger)
    assert named_entities_counter.transform(
        nlp_test_data.NER_DOCS) == expected_results



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

import numpy as np

from nltk.tokenize import word_tokenize
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.feature_extraction.text import TfidfVectorizer


class WordEmbedsDocVectorizer(BaseEstimator, TransformerMixin):
    """
    Converts documents to word2vec based document vector representations.
    It maps the words of a document to word2vec vectors, and averages them
    across dimensions to produce a document vector representation.
    """

    def __init__(self, word2vec, tfidf_weights=False):
        """
        :param word2vec: a gensim word2vec model
        :param tfidf_weights: If set to True, each word will be weighted according to
                        its tf-idf score, when calculating the document vector
                        representation. By default this is set to False,
                        and all words are assumed to have an equal weight of 1.
        """
        self.word2vec = word2vec
        self.tfidf_weights = tfidf_weights

    def transform(self, X, y=None):
        """
        :param X: a collection of docs
        :param y:
        :return: a np array where each row corresponds to a document, and the number
                 of columns equals the w2v model's dimensions.
        """
        return self._extract_w2v_features(X, self.word2vec, self.tfidf_weights)

    def fit(self, X, y=None):
        return self

    def _extract_w2v_features(self, docs, word2vec, tfidf_weights=False):
        # Convert text to a word2vec vector
        rows = []
        if tfidf_weights:
            # Generate a tfidf matrix that will act as weight for the generated
            # features
            # analyzer set to nltk word tokenize as default ommits token under
            # two characters
            tv = TfidfVectorizer(analyzer=word_tokenize)
            tfidf_matrix = tv.fit_transform(docs).toarray()

            for i in range(0, len(docs)):
                doc_tfidf_scores = self._get_doc_tfidf_scores(
                    tfidf_matrix, i, tv.get_feature_names())
                vector = self._convert_string2vec(docs[i], doc_tfidf_scores)
                rows.append(vector)
        else:
            for i in range(0, len(docs)):
                vector = self._convert_string2vec(docs[i])
                rows.append(vector)

        return rows

    def _convert_string2vec(self, doc, doc_tfidf_scores=None):
        """
        This method maps each word of a string to a word2vec vector,
        and returns a new vector which is the average of all word vectors in the string.

        :param doc:
        :param doc_tfidf_scores:
        :return: a list with a length that equals the word2vec model's dimensions, and
                where each element represents a dimension of the document.
        """
        # Tokenize string and convert tokens to lowercase
        tokens = word_tokenize(doc)
        tokens = [token.lower() for token in tokens]

        vectors = []  # Holds the word2vec representation for words in a string
        weights = []  # Holds the tfidf weight for words in a string
        if doc_tfidf_scores:
            for token in tokens:
                # Check if the word exists in the loaded word2vec model
                if token in self.word2vec.vocab:
                    vec = self.word2vec.get_vector(token)
                    vectors.append(vec)

                    weight = doc_tfidf_scores.get(token)
                    weights.append(weight)
        else:
            for token in tokens:
                if token in self.word2vec.vocab:
                    vec = self.word2vec.get_vector(token)
                    vectors.append(vec)

                    # Assume equal weight of 1
                    weights.append(1)

        # Todo: Temp fix if no words were found in the word2vec model
        # vocabulaly
        if len(vectors) == 0:
            return list(0 for i in range(0, self.word2vec.vector_size))

        # Return the average of all found word vectors
        return list(np.average(a=vectors, axis=0, weights=weights))

    @staticmethod
    def _get_doc_tfidf_scores(tfidf_matrix, doc_i, vocab):
        """
        This method takes the document's position in the corpus, and the extracted
        vocabulary, and returns a dictionary with words and Tf-idf frequencies.
        This methods joins the feature, id table produced produced by the TfidfVectorizer,
        and the Tf-idf matrix, in which features are encoded with ids.

        :param tfidf_matrix: a tf-idf matrix produced by the TfidfVectorizer
        :param doc_i: the document's index in the corpus
        :param vocab: the extracted vocabulary from the tfidf vectorizer
        :return: a dictionary with words as keys and tf-idf frequencies as values (word -> tf-idf)
        """
        # Find the indexes of all words that exist in the document
        indexes = tfidf_matrix[doc_i, :].nonzero()[0]

        # Fetch all scores from these indexes
        scores = [tfidf_matrix[doc_i, i] for i in indexes]

        # Array of all extracted features (word names)
        names = [vocab[i] for i in indexes]

        return dict(zip(names, scores))

    def _get_column_names(self):
        """
        This method returns the list of dimensions in the format of d1, d2,...dn.
        This list can be used in a pandas dataframe to name the columns according to
        the word2vec dimensions eg.
                df = pd.DataFrame(rows, columns=columns)

        :return: a list with the dimensions of a w2v model in the format of d{n}
        """
        return list(map(lambda i: "d{0}".format(
            i), range(0, self.word2vec.vector_size)))



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

import math

from unittest import mock
import pytest

import numpy as np

from nlpkit.nlp_feature_extraction.word_embeddings_features import WordEmbedsDocVectorizer
import nlpkit.nlp_feature_extraction.tests.test_data.doc_test_data as test_data
import nlpkit.nlp_feature_extraction.tests.test_data.word2vec_test_data as w2v_test_data


@pytest.fixture(scope="module")
def mockWord2vec():
    mock_word2vec = mock.Mock()
    mock_word2vec.vocab = w2v_test_data.DOC
    mock_word2vec.vector_size = w2v_test_data.DIMS
    mock_word2vec.get_vector.side_effect = get_vector_side_effect
    return mock_word2vec


def get_vector_side_effect(*args, **kwargs):
    return w2v_test_data.WORD_VECTORS[args[0]]


def areVectorsEqual(vec_a, vec_b, dims, rel_tol=1e-06):
    """
    Helper function to compare vectors across dimensions with a relative error tolerance

    """
    for i in range(0, dims):
        if not math.isclose(vec_a[i], vec_b[i], rel_tol=rel_tol):
            return False

    return True


class TestString2Vec:
    def test_convert_string2vec_no_weights(self, mockWord2vec):
        wed_vectorizer = WordEmbedsDocVectorizer(mockWord2vec)
        string2vec = wed_vectorizer._convert_string2vec(test_data.DOC_1)
        assert string2vec == w2v_test_data.EXPECTED_WORD2VEC_DOC_REPR

    def test_convert_string2vec_weights(self, mockWord2vec):
        wed_vectorizer = WordEmbedsDocVectorizer(mockWord2vec)
        string2vec = wed_vectorizer._convert_string2vec(
            test_data.DOC_1,
            w2v_test_data.TFIDF_WEIGHTS)
        assert areVectorsEqual(
            string2vec,
            w2v_test_data.EXPECTED_WEIGHTED_WORD2VEC_DOC_REPR,
            w2v_test_data.DIMS)


class TestGetDocTfidfScores:
    def test_get_doc_tfidf_scores(self):
        np_tf_idf_matrix = np.array(test_data.TFIDF_MATRIX_DOCS)
        scores_dict_doc_1 = WordEmbedsDocVectorizer._get_doc_tfidf_scores(
            np_tf_idf_matrix, 0, test_data.TFIDF_MATRIX_VOCAB)

        assert scores_dict_doc_1.get(
            'dog') == test_data.TFIDF_MATRIX_DOCS[0][test_data.TFIDF_MATRIX_VOCAB.index('dog')]
        assert scores_dict_doc_1.get(
            'fox') == test_data.TFIDF_MATRIX_DOCS[0][test_data.TFIDF_MATRIX_VOCAB.index('fox')]

        scores_dict_doc_2 = WordEmbedsDocVectorizer._get_doc_tfidf_scores(
            np_tf_idf_matrix, 1, test_data.TFIDF_MATRIX_VOCAB)
        assert scores_dict_doc_2.get(
            'sausage') == test_data.TFIDF_MATRIX_DOCS[1][test_data.TFIDF_MATRIX_VOCAB.index('sausage')]


class TestWordEmbedsDocVectorizer:
    def test_WordEmbedsDocVectorizer_no_weights(self, mockWord2vec):
        wed_vectorizer = WordEmbedsDocVectorizer(mockWord2vec)
        doc_embeds_df = wed_vectorizer.transform(test_data.DOCS)

        # Test number of docs
        assert len(doc_embeds_df) == 2

        # Test number of dimensions of the resulting doc vectors
        assert len(doc_embeds_df[0]) == w2v_test_data.DIMS

        # Test the doc embedding of the first doc
        assert doc_embeds_df[0] == w2v_test_data.EXPECTED_WORD2VEC_DOC_REPR



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

import os

from examples.example_corpus import corpus

from nlpkit.nlp_feature_extraction import LIWCExtractor
from nlpkit.nlp_feature_extraction.liwc import Liwc

# Replace with the path of a liwc (.dic) file
LIWC_FILEPATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '..', 'nlp_feature_extraction', 'liwc', 'resources',
                 'mock_liwc.dic'))

if __name__ == "__main__":
    liwc = Liwc(LIWC_FILEPATH)
    liwc_extractor = LIWCExtractor(liwc)
    X = liwc_extractor.fit_transform(corpus)

    print(X)

    # Example usage in a pipeline
    # pipeline = Pipeline([
    #     ('liwc', LIWCExtractor(liwc_dict)),
    #     ('vec', DictVectorizer()),
    #     ('norm', StandardScaler(with_mean=False)),
    #     ('clf', SVC(kernel='linear', probability=True))
    # ])



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from examples.example_corpus import corpus
from nlpkit.nlp_feature_extraction import TextStatsExtractor


if __name__ == "__main__":
    # TextStatsExtractor
    stats_extractor = TextStatsExtractor()
    X = stats_extractor.fit_transform(corpus)

    print(X)

    # Example usage in a pipeline
    # pipeline = Pipeline([
    #     ('stats', TextStatsExtractor()),
    #     ('vec', DictVectorizer()),
    #     ('imp', Imputer(strategy='mean', axis=0)),
    #     ('norm', Normalizer()),
    #     ('clf', SVC(kernel='linear', probability=True))
    # ])


import os
import setuptools

HERE = os.path.abspath(os.path.dirname(__file__))

with open("README.md", "r") as fh:
    long_description = fh.read()

with open(os.path.join(HERE, 'requirements.txt'), "r") as fp:
    install_reqs = fp.read().splitlines()

setuptools.setup(
    name="nlpkit-ml",
    version="1.0.0",
    author="Evan Lalopoulos",
    author_email="evan.lalopoulos.2017@my.bristol.ac.uk",
    description="A library of scikit compatible text transformers, that are ready to be integrated in an NLP pipeline "
                "for various classification tasks.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/evanll/nlpkit-ml",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=install_reqs
)



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from collections import Counter

from unittest import mock
import pytest

from nlpkit.nlp_feature_extraction.pos_features import POSExtractor
from nlpkit.nlp_feature_extraction.pos_features import POSTagPreprocessor
import nlpkit.nlp_feature_extraction.tests.test_data.doc_test_data as test_data
import nlpkit.nlp_feature_extraction.tests.test_data.nlp_test_data as nlp_test_data


@pytest.fixture(scope="module")
def mockTagger():
    mock_tagger = mock.Mock()
    mock_tagger.tag.side_effect = tag_side_effect
    return mock_tagger


def tag_side_effect(*args, **kwargs):
    if args[0] == test_data.DOC_1.split(' '):
        return nlp_test_data.POS_TAGS_DOC_1
    elif args[0] == test_data.DOC_2.split(' '):
        return nlp_test_data.POS_TAGS_DOC_2


def test_postag_preprocessor(mockTagger):
    expected_results = [
        'The_DT quick_JJ brown_JJ fox_NN jumped_VBZ over_IN the_DT lazy_JJ dog_NN ._.',
        'My_PRP dog_NN also_RB likes_VBZ eating_JJ sausage_NN ._.'
    ]

    pos_tag_preprocessor = POSTagPreprocessor(mockTagger)
    assert pos_tag_preprocessor.transform(test_data.DOCS) == expected_results


def test_pos_extractor(mockTagger):
    expected_results = [
        Counter({
            'DT': 2,
            'JJ': 3,
            'NN': 2,
            'VBZ': 1,
            'IN': 1,
            '.': 1
        }),
        Counter({
            'PRP': 1,
            'NN': 2,
            'RB': 1,
            'VBZ': 1,
            'JJ': 1,
            '.': 1
        }),
    ]

    pos_extractor = POSExtractor(mockTagger)
    assert pos_extractor.transform(test_data.DOCS) == expected_results



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from nltk.parse.corenlp import CoreNLPParser
from examples.example_corpus import corpus_ner
from nlpkit.nlp_feature_extraction import NERPreprocessor, NamedEntitiesCounter


if __name__ == "__main__":
    sf_parser = CoreNLPParser(url='http://localhost:9000/', tagtype='ner')

    # NERPreprocessor
    ner_preprocessor = NERPreprocessor(sf_parser)
    X = ner_preprocessor.fit_transform(corpus_ner)

    print(X)

    # NamedEntitiesCounter
    named_entities_counter = NamedEntitiesCounter(sf_parser)
    X_c = named_entities_counter.fit_transform(corpus_ner)

    print(X_c)

    # Example usage in a pipeline
    # pipeline = Pipeline([
    #     ('ner', NERPreprocessor(sf_parser)),
    #     ('vec', TfidfVectorizer()),
    #     ('clf', BernoulliNB())
    # ])



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

import os
from collections import Counter

from nlpkit.nlp_feature_extraction.liwc import Liwc
from nlpkit.nlp_feature_extraction import LIWCExtractor

abs_path = os.path.abspath(os.path.dirname(__file__))
LIWC_PATH = os.path.join(
    abs_path,
    '../liwc/resources/mock_liwc.dic')

DOCS = [
    'I love my dog.',
    'I hate fake news.'
]

liwc = Liwc(LIWC_PATH)


def test_liwc_extractor():
    expected_results = [
        Counter({
            'verb': 0.25,
            'present': 0.25,
            'affect': 0.25,
            'posemo': 0.25,
            'bio': 0.25,
            'sexual': 0.25,
            'social': 0.25,
            'funct': 0.25,
            'pronoun': 0.25,
            'ppron': 0.25,
            'i': 0.25
        }),
        Counter({
            'affect': 0.5,
            'negemo': 0.5,
            'verb': 0.25,
            'present': 0.25,
            'anger': 0.25
        })
    ]

    liwc_extractor = LIWCExtractor(liwc)

    assert liwc_extractor.transform(DOCS) == expected_results


def test_liwc_extractor_with_filters():
    expected_results = [
        Counter({
            'posemo': 0.25
        }), Counter({
            'negemo': 0.5
        })]

    liwc_extractor = LIWCExtractor(liwc, ['posemo', 'negemo'])

    assert liwc_extractor.transform(DOCS) == expected_results



# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

corpus = [
    'The quick brown fox jumped over the lazy dog.',
    'My dog also likes eating sausage.'
]

corpus_ner = [
    "Hillary Clinton agrees with John McCain \"by voting to give George Bush the benefit of the doubt on Iran.\"",
    "My dog also likes eating sausage."
]


# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from nltk.parse.corenlp import CoreNLPParser

from examples.example_corpus import corpus

from nlpkit.nlp_feature_extraction import POSExtractor
from nlpkit.nlp_feature_extraction import POSTagPreprocessor


if __name__ == "__main__":
    sf_parser = CoreNLPParser(url='http://localhost:9000/', tagtype='pos')

    # POSExtractor
    pos_extractor = POSExtractor(sf_parser)
    X = pos_extractor.fit_transform(corpus)

    print(X)

    # POSTagPreprocessor
    pos_preprocessor = POSTagPreprocessor(sf_parser)
    X_pre = pos_preprocessor.fit_transform(corpus)

    print(X_pre)

    # Example usage in a pipeline
    # pipeline = Pipeline([
    #     ('pos', POSExtractor(tagger=sf_parser)),
    #     ('dict_vec', DictVectorizer()),
    #     ('norm', Normalizer()),
    #     ('clf', SVC(kernel='linear', C=1, probability=True))
    # ])


# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

from sklearn.base import BaseEstimator, TransformerMixin


class LIWCExtractor(BaseEstimator, TransformerMixin):
    """
    Extracts proportions of words that fall in the various LIWC categories for a collection of text documents
    """

    def __init__(self, liwc, category_filter=None):
        """
        :param liwc: a LIWC object
        :param category_filter: a set of LIWC categories of interest to act as filter.
                                If a filter is not specified, word proportions for all
                                categories will be extracted.
        """
        self.liwc = liwc
        self.category_filter = category_filter

    def transform(self, X, y=None):
        """
        :param X: a list of docs
        :param y:
        :return: A list of dictionaries (category -> word proportions) corresponding to each doc
        """
        return self.__count_liwc_categories(X, self.liwc, self.category_filter)

    def fit(self, X, y=None):
        return self

    def __count_liwc_categories(self, docs, liwc, category_filter):
        docs_liwc_cats = []  # Holds a dict with category -> word proportions for each doc

        for doc in docs:
            tokens = doc.split()

            # All categories found in the document with raw word counts
            cats = liwc.parse(tokens)

            # Calculate proportions in each category
            for cat, count in cats.items():
                cats[cat] = count / len(tokens)

            if category_filter is not None:
                # Substract filtered categories from all categories, and remove
                # unwanted categories from the dict
                all_cats = set(liwc.categories.values())
                unwanted_cats = all_cats.difference(category_filter)
                for unwanted_cat in unwanted_cats:
                    # Use None to catch errors if key not exists
                    cats.pop(unwanted_cat, None)

            docs_liwc_cats.append(cats)

        return docs_liwc_cats
